﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using XrMapLib;

namespace SHP2SQL
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FillFieldList()
        {
            lbFields.Items.Clear();

            if (axXr1.Layers.IsLayerExist("map"))
            {
                IShapeMapLayer lyr = axXr1.Layers.GetLayerAsShapeMap("map");
                IAttributeTable attTbl = lyr.AttributeTable;
                IFieldSet fs = attTbl.FieldSet;

                int cntFields = fs.FieldCount;
                for (int iField = 0; iField < cntFields; ++iField)
                {
                    XrFieldTypeEnum ft = fs.GetFieldType(iField);
                    if (ft == XrFieldTypeEnum.XrInteger || ft == XrFieldTypeEnum.XrShort || ft == XrFieldTypeEnum.XrVeryShort)
                    {
                        lbFields.Items.Add(fs.GetFieldName(iField) + " - INT");
                    }
                    else if (ft == XrFieldTypeEnum.XrFloat || ft == XrFieldTypeEnum.XrDouble)
                    {
                        lbFields.Items.Add(fs.GetFieldName(iField) + " - DOUBLE");
                    }
                    else
                    {
                        lbFields.Items.Add(fs.GetFieldName(iField) + " - TEXT");
                    }

                    lbFields.SetItemChecked(iField, true);
                }
            }
        }

        private void btnSHP_Click(object sender, EventArgs e)
        {
            openFileDialog.FileName = "";
            openFileDialog.Filter = "SHP Files(*.shp)|*.shp|All Files(*.*)|*.*";
            openFileDialog.FilterIndex = 0;
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbSHP.Text = openFileDialog.FileName;

                axXr1.Layers.RemoveAllLayer();
                axXr1.Layers.AddShapeMapLayer("map", openFileDialog.FileName);
                axXr1.WaitForAllConnections();

                FillFieldList();
            }
        }

        private void btnTXT_Click(object sender, EventArgs e)
        {
            saveFileDialog.FileName = "";
            saveFileDialog.Filter = "SQL Files(*.sql)|*.sql|All Files(*.*)|*.*";
            saveFileDialog.FilterIndex = 0;
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbCREATESQL.Text = saveFileDialog.FileName;
            }
        }

        private String GetInsertPrefixSQL(String strTableName, bool bPointType)
        {
            String result = "INSERT INTO " + strTableName + " (";

            IShapeMapLayer lyr = axXr1.Layers.GetLayerAsShapeMap("map");
            IAttributeTable attTbl = lyr.AttributeTable;
            IFieldSet fs = attTbl.FieldSet;

            int cntFields = fs.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                if (lbFields.GetItemChecked(iField))
                {
                    result += fs.GetFieldName(iField) + ", ";
                }
            }

            if (cbExportFID.Checked)
            {
                result += "FID, ";
            }

            if (bPointType)
            {
                result += "X, Y) VALUES (";
            }
            else
            {
                result += "MinX, MinY, MaxX, MaxY) VALUES (";
            }

            return result;
        }

        private String GetCreateTableSQL(String strTableName, bool bPointType)
        {
            String result = "CREATE TABLE " + strTableName + " (";

            IShapeMapLayer lyr = axXr1.Layers.GetLayerAsShapeMap("map");
            IAttributeTable attTbl = lyr.AttributeTable;
            IFieldSet fs = attTbl.FieldSet;

            int cntFields = fs.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                if (lbFields.GetItemChecked(iField))
                {
                    XrFieldTypeEnum ft = fs.GetFieldType(iField);
                    if (ft == XrFieldTypeEnum.XrInteger || ft == XrFieldTypeEnum.XrShort || ft == XrFieldTypeEnum.XrVeryShort)
                    {
                        result += fs.GetFieldName(iField) + " INTEGER, ";
                    }
                    else if (ft == XrFieldTypeEnum.XrFloat || ft == XrFieldTypeEnum.XrDouble)
                    {
                        result += fs.GetFieldName(iField) + " DOUBLE PRECISION, ";
                    }
                    else
                    {
                        result += fs.GetFieldName(iField) + " CHAR(255), ";
                    }
                }
            }

            if (cbExportFID.Checked)
            {
                result += "FID INTEGER, ";
            }

            if (bPointType) result += "X DOUBLE PRECISION, Y DOUBLE PRECISION);\n\n";
            else result += "MinX DOUBLE PRECISION, MinY DOUBLE PRECISION, MaxX DOUBLE PRECISION, MaxY DOUBLE PRECISION);\n\n";

            return result;
        }

        private void btnConvert_Click(object sender, EventArgs e)
        {
            String strTableName = tbTable.Text.Trim();
            if (strTableName.Length == 0)
            {
                MessageBox.Show("저장할 테이블 명을 입력해 주세요.");
                return;
            }
            
            if (axXr1.Layers.IsLayerExist("map"))
            {
                IShapeMapLayer lyr = axXr1.Layers.GetLayerAsShapeMap("map");
                IShapeTable shpTbl = lyr.ShapeTable;
                IAttributeTable attTbl = lyr.AttributeTable;
                IFieldSet fs = attTbl.FieldSet;

                FileStream fo = null;
                StreamWriter so = null;

                // CREATE SQL FILE 
                //--------------------
                try
                {
                    fo = new FileStream(tbCREATESQL.Text, FileMode.Create);
                    so = new StreamWriter(fo, System.Text.Encoding.Default);
                }
                catch (Exception)
                {
                    MessageBox.Show("저장할 CREATE SQL 파일을 옳바르게 지정해 주세요.");
                    return;
                }

                bool bPointType = (shpTbl.ShapeType == XrShapeTypeEnum.XrPoint);
                String sqlCreateTable = GetCreateTableSQL(strTableName, bPointType);
                so.Write(sqlCreateTable);

                so.Close();
                //--------------------

                // INSERT SQL FILE 
                //--------------------
                try
                {
                    fo = new FileStream(tbINSERTSQL.Text, FileMode.Create);
                    so = new StreamWriter(fo, System.Text.Encoding.Default);
                }
                catch (Exception)
                {
                    MessageBox.Show("저장할 INSERT SQL 파일을 옳바르게 지정해 주세요.");
                    return;
                }

                String sqlInsertRowPrefix = GetInsertPrefixSQL(strTableName, bPointType);

                int FID = 0;
                IShapeRow sRow = null;
                IAttributeRow aRow = null;
                int cntFields = attTbl.FieldSet.FieldCount;
                while ((sRow = shpTbl.GetRow(FID)) != null)
                {
                    String sqlInsertRow = sqlInsertRowPrefix;

                    aRow = attTbl.GetRow(FID);
                    if (aRow.Load())
                    {
                        for(int iField=0; iField<cntFields; ++iField)
                        {
                            if (lbFields.GetItemChecked(iField))
                            {
                                String value = aRow.GetValueAsString(iField);
                                XrFieldTypeEnum ft = fs.GetFieldType(iField);
                                
                                if (ft == XrFieldTypeEnum.XrInteger || ft == XrFieldTypeEnum.XrShort || ft == XrFieldTypeEnum.XrVeryShort)
                                {
                                    sqlInsertRow += value + ", ";
                                }
                                else if (ft == XrFieldTypeEnum.XrFloat || ft == XrFieldTypeEnum.XrDouble)
                                {
                                    sqlInsertRow += value + ", ";
                                }
                                else
                                {
                                    sqlInsertRow += "\'" + value + "\', ";
                                }
                            }
                        }

                        aRow.Unload();
                    }

                    if (cbExportFID.Checked)
                    {
                        sqlInsertRow += FID + ", ";
                    }

                    if (bPointType)
                    {
                        if (sRow.Load())
                        {
                            XrMapLib.Coordinate coord = sRow.GetVertex(0);
                            sqlInsertRow += coord.X + ", " + coord.Y + ");\n\n";
                            sRow.Unload();
                        }
                        else
                        {
                            MessageBox.Show("포인트의 좌표를 읽을 수 없습니다.");
                            return;
                        }
                    }
                    else
                    {
                        Extent MBR = sRow.Extent;
                        sqlInsertRow += MBR.MinX + ", " + MBR.MinY + ", " + MBR.MaxX + ", " + MBR.MaxY + ");\n\n";
                    }

                    FID++;

                    so.Write(sqlInsertRow);
                }

                so.Close();
                //--------------------

                MessageBox.Show("완료");
            }
            else
            {
                MessageBox.Show("SHP 파일을 옳바르지 지정해주세요.");
            }
        }

        private void btnINSERTSQL_Click(object sender, EventArgs e)
        {
            saveFileDialog.FileName = "";
            saveFileDialog.Filter = "SQL Files(*.sql)|*.sql|All Files(*.*)|*.*";
            saveFileDialog.FilterIndex = 0;
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbINSERTSQL.Text = saveFileDialog.FileName;
            }
        }
    }
}